<?php

class event_cal_data {
  
  public $events = array();
  public $event_id = '';
  public $id_list = array();
  public $db = '';
  
  function __construct() {
    
    $this->db = rex_sql::factory();
    
  }
  
  /**
  * !!! Notwendig für korrekte Initialisierung
  */
  function get($id) {
  
    $this->event_id = $id;

    // Existierender Eintrag (noch ungeprüft)
    // Folge ungültiger ID noch unklar  
    if ( isset($this->events[$id]) ) {
      return array($id => $this->events[$id]);
    }

    // Neuer Eintrag
    if ( $id == -1) {
      $events[$id] = self::create_new();
      $this->events = array($id => $events[$id]);
      return array($id => $events[$id]);
    }
    
    // Eintrag existiert noch nicht
    if ( $id != -1 and !isset($this->events[$id]) ) {

      $sql = rex_sql::factory();
      $sql->prepareQuery('
        SELECT * FROM '.rex::getTable("event_cal_events").' WHERE id = :id
      ');
      $sql->execute( array(':id' => $id) );
    
      $data = $sql->getArray();
      $this->events[$data[key($data)]['id']] = $data[0];    
      $this->id_list[] = $data[key($data)]['id'];     
      
    }
    
    
  }

  /**
  * Weiterleitung an rex_sql->setQuery()
  * Example1: $sql->setQuery('SELECT * FROM mytable where id=:id, 'array('id' => 3));
  */
  function setQuery($query, $params) {
    
    return $this->db->setQuery($query, $params);
    
  }
  
  /**
  * @param <String> SQL Statement in PDO::prepare Form
  * @param <Array> Array wie es für die PDO::prepare Form notwendig
  */
  function get_prepared_list($prepared, $arr) {
        
  }

  function get_intervall_list($year=0, $month=0, $section=0) {
    
    if ( $year == 0 ) {
      $year = date('Y');
    } else {
      $year = event_cal_sanitize::sanitizeEventTime($year.'-01-01', "Y");
    }
    
    if ( $month == 0 ) {
      $month = date('m');
    } else {
      $month = event_cal_sanitize::sanitizeEventTime('2016-'.$month.'-01', "m");
    }
        
    $this->db->prepareQuery('
      SELECT * FROM '.rex::getTable("event_cal_events").'
      WHERE time_begin >= :start AND time_begin <= :end
      ORDER BY time_begin ASC
    ');
    $this->db->execute( array(
        ':start' => $year.'-'.$month.'-01',
        ':end' => $year.'-'.$month.'-31'
      ) 
    );
        
    foreach ($this->db->getArray() as $key => $value) {
    
      $this->events[$value['id']] = $value;    
      $this->id_list[] = $value['id'];
            
    }
  
  }
  
  /**
  * Liefert die erste ID
  * oder 0
  */
  function firstId() {
    
    if ( is_numeric($this->id_list[0]) ) {
      return $this->id_list[0];
    } else {
      return 0;
    }
    
  }
  
  /**
  * Liefert die nächste ID falls Liste 
  * oder 0 (keine ID)
  * @return <int>
  */ 
  function nextId() {
    
    $current_id = $this->getId();
    $current_pos = array_search ( $current_id , $this->id_list );
    $next_pos = $current_pos + 1;
    
    if ( is_numeric($this->id_list[$next_pos]) ) {
      return $this->id_list[$next_pos];
    } else {
      return 0;
    }
    
  }
  
  function create_new() {
    
    $eventdata['id'] = -1;
    $eventdata['title'] = '';
    $eventdata['eventjson'] = '';
    $eventdata['time_begin'] = date("Y-m-d H:i");
    $eventdata['time_end'] = date("Y-m-d H:i");;
    $eventdata['tags'] = '';           
    $eventdata['weekday'] = '';
    
    return $eventdata;
        
  }
      
  function getId() {
    
    return $this->event_id;
    
  }
  
  function getTitle() {
    
    return $this->events[$this->getId()]['title'];
    
  }
  
  function getDescription() {
    
    $json = json_decode($this->events[$this->getId()]['eventjson']);
    
    return $json->desc;
    
  }

  function getTimeStart() {
    
    return date("Y-m-d H:i", strtotime($this->events[$this->getId()]['time_begin']));
    
  }

  function getTimeEnd() {
    
    return date("Y-m-d H:i", strtotime($this->events[$this->getId()]['time_end']));
    
  }
  
  function getTags() {
    
    $tags_as_array = explode(",", $this->events[$this->getId()]['tags']);
    $tags_as_string = implode(", ", $tags_as_array);
        
    return $tags_as_string;
    
  }

  /**
  * Liefert ein [Array] mit einer Tag-Liste und deren Status
  */
  function getTagsStatus () {
    
    // Zugewiesene Tags
    $tags = explode(",", $this->events[$this->getId()]['tags']);
    // Tags die nach Konfiguration zugewiesen sein dürfen
    $cfg_tags = explode(",", rex_addon::get('event_cal')->getConfig('tags') );
    // Gesamte Tagliste  
    $taglist = array();
    // Dient zum Vergleich beider Tag-Arrays
    $difftag = array();
    
    foreach ($cfg_tags as $k => $v) {

        $difftag[] = trim($v);
    
        if ( in_array(trim($v), $tags) ) {
            $check = 'checked';
        } else {
            $check = '';
        }
        
        if ( trim($v) != '' ) {
          $taglist[] = array ('name' => trim($v), 'checked' => $check);
        }
        
    }
    
    // Tags aus der Konfiguration werden mit den Tags des Ereignisses verglichen und letztere ergänzt
    foreach ($tags as $k => $v) {
      
      if ( trim($v) != '' and !in_array(trim($v), $difftag) ) {
        $taglist[] = array ('name' => trim($v), 'checked' => 'checked');
      }
      
    }
    
    return $taglist;
      
  }
  
  /**
  * Liefert ein Array mit den Event IDs in der Reihenfolge der Abholung
  */
  function getListOrder() {
    
    return $this->id_list;
    
  }

  function set_event($data) {
    
    // Überprüfung auf korrekte Eingaben fehlt noch
    // eventuell auch in Funktion davor sinnvoller
    
    $id = key($data);
    
    if ( isset($data[$id]['tags']) ) {
      $data[$id]['tags'] = implode(",", $data[$id]['tags']);
    } else {
      $data[$id]['tags'] = '';
    }
    
    // Konvertiert variable Inhalte die nicht durchsucht werden in JSON
    $json['desc'] = $data[$id]['desc'];
    $data[$id]['json'] = json_encode( $json );
    
    if ( $id == -1 ) {
      
      $sql = rex_sql::factory();
      $sql->prepareQuery('
        INSERT INTO '.rex::getTable("event_cal_events").' 
          (title, eventjson, time_begin, time_end, tags)
        VALUES 
          (:title, :eventjson, :time_begin, :time_end, :tags)        
      ');
            
      $sql->execute( array(
        ':title' => $data[$id]['title'], 
        ':eventjson' => $data[$id]['json'], 
        ':time_begin' => event_cal_sanitize::sanitizeEventTime($data[$id]['start'], "Y-m-d H:i"), 
        ':time_end' => event_cal_sanitize::sanitizeEventTime($data[$id]['end'], "Y-m-d H:i"), 
        ':tags' => $data[$id]['tags']
        )
      );
      
      // Gibt die neue Ereignis ID zurück
      // damit kann z.B. die richtige Stelle farblich markiert und dorthin gesprungen werden
      $id = $sql->getLastId();
      
    } else {
             
      $sql = rex_sql::factory();
      $sql->prepareQuery('
        UPDATE '.rex::getTable("event_cal_events").' 
        SET title=:title, eventjson=:eventjson, time_begin=:time_begin, time_end=:time_end, tags=:tags
        WHERE id = :id; 
      ');
      
      $sql->execute( array(
        ':title' => $data[$id]['title'], 
        ':eventjson' => $data[$id]['json'], 
        ':time_begin' => event_cal_sanitize::sanitizeEventTime($data[$id]['start'], "Y-m-d H:i"), 
        ':time_end' => event_cal_sanitize::sanitizeEventTime($data[$id]['end'], "Y-m-d H:i"), 
        ':tags' => $data[$id]['tags'], 
        ':id' => $id
        ) 
      );
      
      // Gibt die neue Ereignis ID zurück
      // damit kann z.B. die richtige Stelle farblich markiert und dorthin gesprungen werden
      $id = $id;
           
    }
    
    $this->get($id);
    
  }
  
  function del_event($id) {
    
    $this->get($id);
    
    $this->db->prepareQuery('
      DELETE FROM '.rex::getTable("event_cal_events").' 
      WHERE id = :id
    ');
    
    $this->db->execute( array(':id' => $id) );
        
  }  
  
}

?>